
/*
    Для управления ШД в программе используется библиотека AccelStepper - http://www.open.com.au/mikem/arduino/AccelStepper/index.html
*/

#include <AccelStepper.h>

#define MOTOR1_DIR_PIN  8
#define MOTOR2_DIR_PIN  11

#define MOTOR1_STEP_PIN 9
#define MOTOR2_STEP_PIN 10

#define MOTOR1_LS_PIN 6
#define MOTOR2_LS_PIN 7

#define MOTOR1_SPR 200  // Кол-во шагов на 1 оборот вала двигателя
#define MOTOR2_SPR 200  

#define MOTOR1_MAX_SPEED 50000.0
#define MOTOR2_MAX_SPEED 50000.0

#define MOTOR1_INIT_SPEED -200.0
#define MOTOR2_INIT_SPEED 800.0

#define MOTOR1_JOY_IN 0 // Ось Y джойстика
#define MOTOR2_JOY_IN 1 // Ось X джойстика
#define SPEED_KNOB_IN 2 // Задатчик скорости - переменный резистор 4,7К

AccelStepper stpMotor1(AccelStepper::DRIVER, MOTOR1_STEP_PIN, MOTOR1_DIR_PIN);
AccelStepper stpMotor2(AccelStepper::DRIVER, MOTOR2_STEP_PIN, MOTOR2_DIR_PIN);

int nMotor1SP, nMotor2SP;            // Задание направления от джойстика
int nSpeedSP;                        // Задание скорости от задачика
int nMotor1MV, nMotor2MV;  // Задание скорости после ограничителя

void setup() {                
  phInit();
}

void loop() {
  phGetCommand();
  phMotorRun();
}


void phInit()
{
  pinMode(MOTOR1_DIR_PIN, OUTPUT);
  pinMode(MOTOR2_DIR_PIN, OUTPUT);
  pinMode(MOTOR1_STEP_PIN, OUTPUT);
  pinMode(MOTOR2_STEP_PIN, OUTPUT);
 
  pinMode(MOTOR1_LS_PIN, INPUT_PULLUP);
  pinMode(MOTOR2_LS_PIN, INPUT_PULLUP);
  
  nMotor1MV = 0;
  nMotor2MV = 0;
  stpMotor1.setMaxSpeed(MOTOR1_MAX_SPEED);
  stpMotor2.setMaxSpeed(MOTOR2_MAX_SPEED);
  stpMotor1.setAcceleration(10.0);
  stpMotor2.setAcceleration(10.0);
  stpMotor1.setSpeed(0);
  stpMotor2.setSpeed(0);
  
  stpMotor1.setSpeed(MOTOR1_INIT_SPEED);
  while (digitalRead(MOTOR1_LS_PIN) != 0) {
    stpMotor1.runSpeed();
  }

  stpMotor1.setCurrentPosition(0);
  stpMotor1.setSpeed(-MOTOR1_INIT_SPEED);
  while (abs(stpMotor1.currentPosition()) < 400) {
    stpMotor1.runSpeed();
  } 
  stpMotor1.setCurrentPosition(0);
  stpMotor1.setSpeed(0);

  stpMotor2.setSpeed(MOTOR2_INIT_SPEED);
  while (digitalRead(MOTOR2_LS_PIN) != 0) {
    stpMotor2.runSpeed();
  }

  stpMotor2.setCurrentPosition(0);
  stpMotor2.setSpeed(-MOTOR2_INIT_SPEED);
  while (abs(stpMotor2.currentPosition()) < 2400) {
    stpMotor2.runSpeed();
  } 
  stpMotor2.setCurrentPosition(0);
  stpMotor2.setSpeed(0);
  
  return;
}

void phGetCommand()
{
  nMotor1SP = analogRead(MOTOR1_JOY_IN) - 512;
  nMotor2SP = analogRead(MOTOR2_JOY_IN) - 512;
  nSpeedSP = 1023 - analogRead(SPEED_KNOB_IN);
  
  if (nMotor1SP > 100) {
    nMotor1MV = nSpeedSP;
  } else if (nMotor1SP < -100) {
    nMotor1MV = -nSpeedSP;
  } else {
    nMotor1MV = 0;
  }
  
  nMotor1MV = nMotor1MV / 5;
  
  if (nMotor2SP > 100) {
    nMotor2MV = nSpeedSP;
  } else if (nMotor2SP < -100) {
    nMotor2MV = -nSpeedSP;
  } else {
    nMotor2MV = 0;
  }
  
  nMotor2MV = nMotor2MV * -1;
}

void phMotorRun() 
{
  stpMotor1.setSpeed(nMotor1MV);
  stpMotor2.setSpeed(nMotor2MV);
  
  stpMotor1.runSpeed();
  stpMotor2.runSpeed();  
}
